@@ -8,6 +8,7 @@ import pandas as pd
 from numba import jit
 from typing import Union, Tuple, Optional
 from .base import BaseIndicator
+from .utils import true_range, ema_wilder
 
 
 class ADX(BaseIndicator):
@@ -23,30 +24,22 @@ class ADX(BaseIndicator):
         super().__init__("ADX")
     
     @staticmethod
-    @jit(nopython=True)
-    def _calculate_adx(high: np.ndarray, low: np.ndarray, close: np.ndarray,
+    @jit(nopython=True, cache=True)
+    def _calculate_adx_optimized(high: np.ndarray, low: np.ndarray, close: np.ndarray,
                       period: int) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
-        """Numba optimized ADX calculation"""
+        """Optimized ADX calculation using consolidated utilities"""
         n = len(high)
         
-        # Initialize arrays
-        tr = np.empty(n)
+        # Use consolidated true_range utility
+        tr = true_range(high, low, close)
+        
+        # Calculate Directional Movement
         dm_plus = np.empty(n)
         dm_minus = np.empty(n)
-        
-        # Calculate True Range and Directional Movement
-        tr[0] = high[0] - low[0]
         dm_plus[0] = 0
         dm_minus[0] = 0
         
         for i in range(1, n):
-            # True Range
-            hl = high[i] - low[i]
-            hc = abs(high[i] - close[i-1])
-            lc = abs(low[i] - close[i-1])
-            tr[i] = max(hl, hc, lc)
-            
-            # Directional Movement
             up_move = high[i] - high[i-1]
             down_move = low[i-1] - low[i]
             
@@ -60,51 +53,28 @@ class ADX(BaseIndicator):
             else:
                 dm_minus[i] = 0
         
-        # Calculate smoothed values
-        atr = np.full(n, np.nan)
+        # Use optimized Wilder's smoothing for ATR and DM
+        atr = ema_wilder(tr, period)
+        sm_dm_plus = ema_wilder(dm_plus, period)
+        sm_dm_minus = ema_wilder(dm_minus, period)
+        
+        # Calculate DI values
         di_plus = np.full(n, np.nan)
         di_minus = np.full(n, np.nan)
-        adx = np.full(n, np.nan)
-        dx = np.full(n, np.nan)  # store DX values separately for proper ADX seed
+        dx = np.full(n, np.nan)
         
-        # Initial smoothed values
-        if n >= period:
-            atr[period-1] = np.mean(tr[:period])
-            sm_dm_plus = np.mean(dm_plus[:period])
-            sm_dm_minus = np.mean(dm_minus[:period])
-            
-            if atr[period-1] > 0:
-                di_plus[period-1] = (sm_dm_plus / atr[period-1]) * 100
-                di_minus[period-1] = (sm_dm_minus / atr[period-1]) * 100
-            
-            # Calculate subsequent values
-            for i in range(period, n):
-                # Smoothed TR
-                atr[i] = (atr[i-1] * (period - 1) + tr[i]) / period
+        for i in range(period-1, n):
+            if atr[i] > 0:
+                di_plus[i] = (sm_dm_plus[i] / atr[i]) * 100
+                di_minus[i] = (sm_dm_minus[i] / atr[i]) * 100
                 
-                # Smoothed DM
-                sm_dm_plus = (sm_dm_plus * (period - 1) + dm_plus[i]) / period
-                sm_dm_minus = (sm_dm_minus * (period - 1) + dm_minus[i]) / period
-                
-                # DI calculations
-                if atr[i] > 0:
-                    di_plus[i] = (sm_dm_plus / atr[i]) * 100
-                    di_minus[i] = (sm_dm_minus / atr[i]) * 100
-                
-                # DX calculation (store for later ADX seed)
+                # DX calculation
                 di_sum = di_plus[i] + di_minus[i]
                 if di_sum > 0:
                     dx[i] = abs(di_plus[i] - di_minus[i]) / di_sum * 100
         
-            # ---- ADX Seed & Smoothing ----
-            first_adx_pos = period * 2 - 1
-            if n > first_adx_pos:
-                # Average of DX over the first 'period' values to seed ADX
-                adx[first_adx_pos] = np.nanmean(dx[period:first_adx_pos + 1])
-                # Continue smoothing ADX for remaining periods
-                for i in range(first_adx_pos + 1, n):
-                    if not np.isnan(dx[i]):
-                        adx[i] = (adx[i-1] * (period - 1) + dx[i]) / period
+        # Calculate ADX using Wilder's smoothing of DX
+        adx = ema_wilder(dx, period)
         
         return di_plus, di_minus, adx
     
@@ -138,7 +108,7 @@ class ADX(BaseIndicator):
         high_data, low_data, close_data = self.align_arrays(high_data, low_data, close_data)
         self.validate_period(period, len(close_data))
         
-        results = self._calculate_adx(high_data, low_data, close_data, period)
+        results = self._calculate_adx_optimized(high_data, low_data, close_data, period)
         return self.format_multiple_outputs(results, input_type, index)
 
 
@@ -476,7 +446,7 @@ class PSAR(BaseIndicator):
             return results[0]
 
 
-class HT_TRENDLINE(BaseIndicator):
+class HT(BaseIndicator):
     """
     Hilbert Transform - Instantaneous Trendline
     
@@ -521,4 +491,296 @@ class HT_TRENDLINE(BaseIndicator):
         """
         validated_data, input_type, index = self.validate_input(data)
         result = self._calculate_ht_trendline(validated_data)
-        return self.format_output(result, input_type, index)
\ No newline at end of file
+        return self.format_output(result, input_type, index)
+
+
+class ZigZag(BaseIndicator):
+    """
+    Zig Zag
+    
+    Zig Zag connects significant price swings and filters out smaller price movements.
+    
+    Formula: Connect swing highs and lows that exceed the deviation threshold.
+    """
+    
+    def __init__(self):
+        super().__init__("ZigZag")
+    
+    @staticmethod
+    @jit(nopython=True)
+    def _calculate_zigzag(high: np.ndarray, low: np.ndarray, close: np.ndarray, deviation: float) -> np.ndarray:
+        """Numba optimized Zig Zag calculation"""
+        n = len(close)
+        result = np.full(n, np.nan)
+        
+        if n < 3:
+            return result
+        
+        # Find first significant point
+        last_pivot_idx = 0
+        last_pivot_price = close[0]
+        last_pivot_type = 0  # 0 = unknown, 1 = high, -1 = low
+        result[0] = close[0]
+        
+        current_high = high[0]
+        current_low = low[0]
+        current_high_idx = 0
+        current_low_idx = 0
+        
+        for i in range(1, n):
+            # Update current extremes
+            if high[i] > current_high:
+                current_high = high[i]
+                current_high_idx = i
+            if low[i] < current_low:
+                current_low = low[i]
+                current_low_idx = i
+            
+            # Check for reversal from high
+            if last_pivot_type != -1:  # Not currently looking for high
+                change_from_high = (current_high - low[i]) / current_high * 100
+                if change_from_high >= deviation:
+                    # Reversal from high - mark the high
+                    result[current_high_idx] = current_high
+                    last_pivot_idx = current_high_idx
+                    last_pivot_price = current_high
+                    last_pivot_type = 1
+                    current_low = low[i]
+                    current_low_idx = i
+            
+            # Check for reversal from low  
+            if last_pivot_type != 1:  # Not currently looking for low
+                if current_low > 0:
+                    change_from_low = (high[i] - current_low) / current_low * 100
+                    if change_from_low >= deviation:
+                        # Reversal from low - mark the low
+                        result[current_low_idx] = current_low
+                        last_pivot_idx = current_low_idx
+                        last_pivot_price = current_low
+                        last_pivot_type = -1
+                        current_high = high[i]
+                        current_high_idx = i
+        
+        return result
+    
+    def calculate(self, high: Union[np.ndarray, pd.Series, list],
+                 low: Union[np.ndarray, pd.Series, list],
+                 close: Union[np.ndarray, pd.Series, list],
+                 deviation: float = 5.0) -> Union[np.ndarray, pd.Series]:
+        """
+        Calculate Zig Zag
+        
+        Parameters:
+        -----------
+        high : Union[np.ndarray, pd.Series, list]
+            High prices
+        low : Union[np.ndarray, pd.Series, list]
+            Low prices
+        close : Union[np.ndarray, pd.Series, list]
+            Closing prices
+        deviation : float, default=5.0
+            Minimum percentage deviation for zig zag line
+            
+        Returns:
+        --------
+        Union[np.ndarray, pd.Series]
+            Zig Zag values in the same format as input
+        """
+        high_data, input_type, index = self.validate_input(high)
+        low_data, _, _ = self.validate_input(low)
+        close_data, _, _ = self.validate_input(close)
+        
+        high_data, low_data, close_data = self.align_arrays(high_data, low_data, close_data)
+        
+        result = self._calculate_zigzag(high_data, low_data, close_data, deviation)
+        return self.format_output(result, input_type, index)
+
+
+class WilliamsFractals(BaseIndicator):
+    """
+    Williams Fractals
+    
+    Identifies turning points (fractals) in price action.
+    
+    Fractal Up: High[n] > High[n-2] and High[n] > High[n-1] and High[n] > High[n+1] and High[n] > High[n+2]
+    Fractal Down: Low[n] < Low[n-2] and Low[n] < Low[n-1] and Low[n] < Low[n+1] and Low[n] < Low[n+2]
+    """
+    
+    def __init__(self):
+        super().__init__("Fractals")
+    
+    @staticmethod
+    @jit(nopython=True)
+    def _calculate_fractals(high: np.ndarray, low: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
+        """Numba optimized Williams Fractals calculation"""
+        n = len(high)
+        fractal_up = np.full(n, np.nan)
+        fractal_down = np.full(n, np.nan)
+        
+        for i in range(2, n - 2):
+            # Check for fractal up (peak)
+            if (high[i] > high[i-1] and high[i] > high[i-2] and 
+                high[i] > high[i+1] and high[i] > high[i+2]):
+                fractal_up[i] = high[i]
+            
+            # Check for fractal down (trough)
+            if (low[i] < low[i-1] and low[i] < low[i-2] and 
+                low[i] < low[i+1] and low[i] < low[i+2]):
+                fractal_down[i] = low[i]
+        
+        return fractal_up, fractal_down
+    
+    def calculate(self, high: Union[np.ndarray, pd.Series, list],
+                 low: Union[np.ndarray, pd.Series, list]) -> Union[Tuple[np.ndarray, np.ndarray], Tuple[pd.Series, pd.Series]]:
+        """
+        Calculate Williams Fractals
+        
+        Parameters:
+        -----------
+        high : Union[np.ndarray, pd.Series, list]
+            High prices
+        low : Union[np.ndarray, pd.Series, list]
+            Low prices
+            
+        Returns:
+        --------
+        Union[Tuple[np.ndarray, np.ndarray], Tuple[pd.Series, pd.Series]]
+            (fractal_up, fractal_down) in the same format as input
+        """
+        high_data, input_type, index = self.validate_input(high)
+        low_data, _, _ = self.validate_input(low)
+        
+        high_data, low_data = self.align_arrays(high_data, low_data)
+        
+        fractal_up, fractal_down = self._calculate_fractals(high_data, low_data)
+        
+        results = (fractal_up, fractal_down)
+        return self.format_multiple_outputs(results, input_type, index)
+
+
+class RWI(BaseIndicator):
+    """
+    Random Walk Index (RWI High/Low)
+    
+    Measures how much a security's price movement differs from a random walk.
+    
+    Formula:
+    RWI High = (High - Low[n]) / (ATR * sqrt(n))
+    RWI Low = (High[n] - Low) / (ATR * sqrt(n))
+    """
+    
+    def __init__(self):
+        super().__init__("RWI")
+    
+    @staticmethod
+    @jit(nopython=True)
+    def _calculate_atr(high: np.ndarray, low: np.ndarray, close: np.ndarray, period: int) -> np.ndarray:
+        """Calculate ATR"""
+        n = len(close)
+        tr = np.full(n, np.nan)
+        atr = np.full(n, np.nan)
+        
+        # Calculate True Range
+        tr[0] = high[0] - low[0]
+        for i in range(1, n):
+            tr[i] = max(high[i] - low[i], 
+                       abs(high[i] - close[i - 1]), 
+                       abs(low[i] - close[i - 1]))
+        
+        # Calculate ATR using simple moving average
+        for i in range(period - 1, n):
+            atr[i] = np.mean(tr[i - period + 1:i + 1])
+        
+        return atr
+    
+    @staticmethod
+    @jit(nopython=True)
+    def _calculate_atr_for_rwi(high: np.ndarray, low: np.ndarray, close: np.ndarray, period: int) -> np.ndarray:
+        """Calculate ATR for RWI"""
+        n = len(close)
+        tr = np.full(n, np.nan)
+        atr = np.full(n, np.nan)
+        
+        # Calculate True Range
+        tr[0] = high[0] - low[0]
+        for i in range(1, n):
+            tr[i] = max(high[i] - low[i], 
+                       abs(high[i] - close[i - 1]), 
+                       abs(low[i] - close[i - 1]))
+        
+        # Calculate ATR using simple moving average
+        for i in range(period - 1, n):
+            atr[i] = np.mean(tr[i - period + 1:i + 1])
+        
+        return atr
+    
+    @staticmethod
+    @jit(nopython=True)
+    def _calculate_rwi(high: np.ndarray, low: np.ndarray, close: np.ndarray, period: int) -> Tuple[np.ndarray, np.ndarray]:
+        """Numba optimized RWI calculation"""
+        n = len(close)
+        rwi_high = np.full(n, np.nan)
+        rwi_low = np.full(n, np.nan)
+        
+        # Calculate ATR inline
+        tr = np.full(n, np.nan)
+        atr = np.full(n, np.nan)
+        
+        # Calculate True Range
+        tr[0] = high[0] - low[0]
+        for i in range(1, n):
+            tr[i] = max(high[i] - low[i], 
+                       abs(high[i] - close[i - 1]), 
+                       abs(low[i] - close[i - 1]))
+        
+        # Calculate ATR using simple moving average
+        for i in range(period - 1, n):
+            atr[i] = np.mean(tr[i - period + 1:i + 1])
+        
+        for i in range(period - 1, n):
+            if atr[i] > 0:
+                # RWI High: (High - Low[n periods ago]) / (ATR * sqrt(n))
+                rwi_high[i] = (high[i] - low[i - period + 1]) / (atr[i] * np.sqrt(period))
+                
+                # RWI Low: (High[n periods ago] - Low) / (ATR * sqrt(n))
+                rwi_low[i] = (high[i - period + 1] - low[i]) / (atr[i] * np.sqrt(period))
+            else:
+                rwi_high[i] = 0.0
+                rwi_low[i] = 0.0
+        
+        return rwi_high, rwi_low
+    
+    def calculate(self, high: Union[np.ndarray, pd.Series, list],
+                 low: Union[np.ndarray, pd.Series, list],
+                 close: Union[np.ndarray, pd.Series, list],
+                 period: int = 14) -> Union[Tuple[np.ndarray, np.ndarray], Tuple[pd.Series, pd.Series]]:
+        """
+        Calculate Random Walk Index
+        
+        Parameters:
+        -----------
+        high : Union[np.ndarray, pd.Series, list]
+            High prices
+        low : Union[np.ndarray, pd.Series, list]
+            Low prices
+        close : Union[np.ndarray, pd.Series, list]
+            Closing prices
+        period : int, default=14
+            Period for RWI calculation
+            
+        Returns:
+        --------
+        Union[Tuple[np.ndarray, np.ndarray], Tuple[pd.Series, pd.Series]]
+            (rwi_high, rwi_low) in the same format as input
+        """
+        high_data, input_type, index = self.validate_input(high)
+        low_data, _, _ = self.validate_input(low)
+        close_data, _, _ = self.validate_input(close)
+        
+        high_data, low_data, close_data = self.align_arrays(high_data, low_data, close_data)
+        self.validate_period(period, len(close_data))
+        
+        rwi_high, rwi_low = self._calculate_rwi(high_data, low_data, close_data, period)
+        
+        results = (rwi_high, rwi_low)
+        return self.format_multiple_outputs(results, input_type, index)
\ No newline at end of file
