@@ -5,9 +5,11 @@ OpenAlgo Technical Indicators - Volatility Indicators
 
 import numpy as np
 import pandas as pd
-from numba import jit
+from openalgo.numba_shim import jit
 from typing import Union, Tuple, Optional
 from .base import BaseIndicator
+from .utils import (ema, atr_wilder, true_range, sma, stdev, highest, lowest, 
+                    rolling_sum, ulcer_index_optimized)
 
 
 class ATR(BaseIndicator):
@@ -25,38 +27,7 @@ class ATR(BaseIndicator):
     def __init__(self):
         super().__init__("ATR")
     
-    @staticmethod
-    @jit(nopython=True)
-    def _calculate_atr(high: np.ndarray, low: np.ndarray, close: np.ndarray, 
-                      period: int) -> np.ndarray:
-        """Numba optimized ATR calculation"""
-        n = len(high)
-        tr = np.empty(n)
-        atr = np.full(n, np.nan)
-        
-        # First TR value
-        tr[0] = high[0] - low[0]
-        
-        # Calculate True Range
-        for i in range(1, n):
-            hl = high[i] - low[i]
-            hc = abs(high[i] - close[i-1])
-            lc = abs(low[i] - close[i-1])
-            tr[i] = max(hl, hc, lc)
-        
-        # Calculate ATR using Wilder's smoothing
-        if n >= period:
-            # Initial ATR is simple average
-            sum_tr = 0.0
-            for i in range(period):
-                sum_tr += tr[i]
-            atr[period-1] = sum_tr / period
-            
-            # Subsequent ATR values use Wilder's smoothing
-            for i in range(period, n):
-                atr[i] = (atr[i-1] * (period - 1) + tr[i]) / period
-        
-        return atr
+    # Removed redundant ATR calculation - using consolidated utility
     
     def calculate(self, high: Union[np.ndarray, pd.Series, list],
                  low: Union[np.ndarray, pd.Series, list],
@@ -89,7 +60,7 @@ class ATR(BaseIndicator):
         high_data, low_data, close_data = self.align_arrays(high_data, low_data, close_data)
         self.validate_period(period, len(close_data))
         
-        result = self._calculate_atr(high_data, low_data, close_data, period)
+        result = atr_wilder(high_data, low_data, close_data, period)
         return self.format_output(result, input_type, index)
 
 
@@ -110,31 +81,16 @@ class BollingerBands(BaseIndicator):
         super().__init__("Bollinger Bands")
     
     @staticmethod
-    @jit(nopython=True)
     def _calculate_bollinger_bands(data: np.ndarray, period: int, 
                                   std_dev: float) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
-        """Numba optimized Bollinger Bands calculation"""
-        n = len(data)
-        upper = np.full(n, np.nan)
-        middle = np.full(n, np.nan)
-        lower = np.full(n, np.nan)
+        """O(n) optimized Bollinger Bands calculation using utils"""
+        # Use optimized O(n) utilities
+        middle = sma(data, period)
+        std_values = stdev(data, period)
         
-        for i in range(period - 1, n):
-            # Calculate SMA (middle band)
-            window_data = data[i - period + 1:i + 1]
-            sma = np.mean(window_data)
-            middle[i] = sma
-            
-            # Calculate standard deviation
-            variance = 0.0
-            for j in range(period):
-                diff = window_data[j] - sma
-                variance += diff * diff
-            std = np.sqrt(variance / period)
-            
-            # Calculate upper and lower bands
-            upper[i] = sma + (std_dev * std)
-            lower[i] = sma - (std_dev * std)
+        # Calculate upper and lower bands
+        upper = middle + (std_dev * std_values)
+        lower = middle - (std_dev * std_values)
         
         return upper, middle, lower
     
@@ -191,7 +147,7 @@ def _calculate_ema_keltner(data: np.ndarray, period: int) -> np.ndarray:
     return ema
 
 
-class KeltnerChannel(BaseIndicator):
+class Keltner(BaseIndicator):
     """
     Keltner Channel
     
@@ -303,7 +259,7 @@ class KeltnerChannel(BaseIndicator):
         return self.format_multiple_outputs(results, input_type, index)
 
 
-class DonchianChannel(BaseIndicator):
+class Donchian(BaseIndicator):
     """
     Donchian Channel
     
@@ -320,24 +276,15 @@ class DonchianChannel(BaseIndicator):
         super().__init__("Donchian Channel")
     
     @staticmethod
-    @jit(nopython=True)
     def _calculate_donchian_channel(high: np.ndarray, low: np.ndarray, 
                                    period: int) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
-        """Numba optimized Donchian Channel calculation"""
-        n = len(high)
-        upper = np.full(n, np.nan)
-        lower = np.full(n, np.nan)
-        middle = np.full(n, np.nan)
+        """O(n) optimized Donchian Channel calculation using utils"""
+        # Use optimized O(n) deque-based utilities
+        upper = highest(high, period)
+        lower = lowest(low, period)
         
-        for i in range(period - 1, n):
-            # Highest high over period
-            upper[i] = high[i - period + 1:i + 1].max()
-            
-            # Lowest low over period
-            lower[i] = low[i - period + 1:i + 1].min()
-            
-            # Middle line
-            middle[i] = (upper[i] + lower[i]) / 2.0
+        # Calculate middle line
+        middle = (upper + lower) / 2.0
         
         return upper, middle, lower
     
@@ -372,7 +319,7 @@ class DonchianChannel(BaseIndicator):
         return self.format_multiple_outputs(results, input_type, index)
 
 
-class ChaikinVolatility(BaseIndicator):
+class Chaikin(BaseIndicator):
     """
     Chaikin Volatility
     
@@ -488,13 +435,8 @@ class NATR(BaseIndicator):
         # Calculate ATR
         atr = self._atr.calculate(high, low, close, period)
         
-        # Calculate NATR
-        natr = np.empty_like(atr)
-        for i in range(len(atr)):
-            if close_data[i] != 0:
-                natr[i] = (atr[i] / close_data[i]) * 100
-            else:
-                natr[i] = 0
+        # Calculate NATR using vectorized operations (O(n) optimization)
+        natr = np.where(close_data != 0, (atr / close_data) * 100, 0)
         
         return self.format_output(natr, input_type, index)
 
@@ -512,24 +454,9 @@ class RVI(BaseIndicator):
         super().__init__("RVI")
     
     @staticmethod
-    @jit(nopython=True)
     def _calculate_stdev(data: np.ndarray, period: int) -> np.ndarray:
-        """Calculate rolling standard deviation"""
-        n = len(data)
-        result = np.full(n, np.nan)
-        
-        for i in range(period - 1, n):
-            window = data[i - period + 1:i + 1]
-            mean_val = np.mean(window)
-            
-            variance = 0.0
-            for j in range(period):
-                diff = window[j] - mean_val
-                variance += diff * diff
-            
-            result[i] = np.sqrt(variance / period)
-        
-        return result
+        """O(n) optimized standard deviation using utils"""
+        return stdev(data, period)
     
     @staticmethod
     @jit(nopython=True)
@@ -902,4 +829,475 @@ class MASS(BaseIndicator):
         # Calculate SMA of ratio
         result = self._calculate_sma(ratio, slow_period)
         
-        return self.format_output(result, input_type, index)
\ No newline at end of file
+        return self.format_output(result, input_type, index)
+
+
+class BBPercent(BaseIndicator):
+    """
+    Bollinger Bands %B
+    
+    %B shows where price is in relation to the bands.
+    %B = 1 when price is at the upper band, 0 when at the lower band.
+    
+    Formula: %B = (Close - Lower Band) / (Upper Band - Lower Band)
+    """
+    
+    def __init__(self):
+        super().__init__("BB %B")
+    
+    @staticmethod
+    @jit(nopython=True)
+    def _calculate_sma(data: np.ndarray, period: int) -> np.ndarray:
+        """Calculate SMA"""
+        n = len(data)
+        result = np.full(n, np.nan)
+        
+        for i in range(period - 1, n):
+            result[i] = np.mean(data[i - period + 1:i + 1])
+        
+        return result
+    
+    @staticmethod
+    @jit(nopython=True)
+    def _calculate_stddev(data: np.ndarray, period: int) -> np.ndarray:
+        """Calculate rolling standard deviation"""
+        n = len(data)
+        result = np.full(n, np.nan)
+        
+        for i in range(period - 1, n):
+            window = data[i - period + 1:i + 1]
+            mean_val = np.mean(window)
+            variance = np.mean((window - mean_val) ** 2)
+            result[i] = np.sqrt(variance)
+        
+        return result
+    
+    def calculate(self, data: Union[np.ndarray, pd.Series, list],
+                 period: int = 20, std_dev: float = 2.0) -> Union[np.ndarray, pd.Series]:
+        """
+        Calculate Bollinger Bands %B
+        
+        Parameters:
+        -----------
+        data : Union[np.ndarray, pd.Series, list]
+            Price data (typically closing prices)
+        period : int, default=20
+            Period for moving average and standard deviation
+        std_dev : float, default=2.0
+            Number of standard deviations for the bands
+            
+        Returns:
+        --------
+        Union[np.ndarray, pd.Series]
+            %B values in the same format as input
+        """
+        validated_data, input_type, index = self.validate_input(data)
+        self.validate_period(period, len(validated_data))
+        
+        # Calculate Bollinger Bands
+        sma = self._calculate_sma(validated_data, period)
+        stddev = self._calculate_stddev(validated_data, period)
+        
+        upper_band = sma + (stddev * std_dev)
+        lower_band = sma - (stddev * std_dev)
+        
+        # Calculate %B
+        percent_b = np.full_like(validated_data, np.nan)
+        for i in range(len(validated_data)):
+            if upper_band[i] != lower_band[i]:
+                percent_b[i] = (validated_data[i] - lower_band[i]) / (upper_band[i] - lower_band[i])
+            else:
+                percent_b[i] = 0.5
+        
+        return self.format_output(percent_b, input_type, index)
+
+
+class BBWidth(BaseIndicator):
+    """
+    Bollinger Bandwidth
+    
+    Bollinger Bandwidth measures the width of the Bollinger Bands.
+    Used to identify periods of low volatility (squeeze) and high volatility.
+    
+    Formula: Bandwidth = (Upper Band - Lower Band) / Middle Band
+    """
+    
+    def __init__(self):
+        super().__init__("BB Bandwidth")
+    
+    @staticmethod
+    @jit(nopython=True)
+    def _calculate_sma(data: np.ndarray, period: int) -> np.ndarray:
+        """Calculate SMA"""
+        n = len(data)
+        result = np.full(n, np.nan)
+        
+        for i in range(period - 1, n):
+            result[i] = np.mean(data[i - period + 1:i + 1])
+        
+        return result
+    
+    @staticmethod
+    @jit(nopython=True)
+    def _calculate_stddev(data: np.ndarray, period: int) -> np.ndarray:
+        """Calculate rolling standard deviation"""
+        n = len(data)
+        result = np.full(n, np.nan)
+        
+        for i in range(period - 1, n):
+            window = data[i - period + 1:i + 1]
+            mean_val = np.mean(window)
+            variance = np.mean((window - mean_val) ** 2)
+            result[i] = np.sqrt(variance)
+        
+        return result
+    
+    def calculate(self, data: Union[np.ndarray, pd.Series, list],
+                 period: int = 20, std_dev: float = 2.0) -> Union[np.ndarray, pd.Series]:
+        """
+        Calculate Bollinger Bandwidth
+        
+        Parameters:
+        -----------
+        data : Union[np.ndarray, pd.Series, list]
+            Price data (typically closing prices)
+        period : int, default=20
+            Period for moving average and standard deviation
+        std_dev : float, default=2.0
+            Number of standard deviations for the bands
+            
+        Returns:
+        --------
+        Union[np.ndarray, pd.Series]
+            Bandwidth values in the same format as input
+        """
+        validated_data, input_type, index = self.validate_input(data)
+        self.validate_period(period, len(validated_data))
+        
+        # Calculate Bollinger Bands
+        sma = self._calculate_sma(validated_data, period)
+        stddev = self._calculate_stddev(validated_data, period)
+        
+        upper_band = sma + (stddev * std_dev)
+        lower_band = sma - (stddev * std_dev)
+        
+        # Calculate Bandwidth
+        bandwidth = np.full_like(validated_data, np.nan)
+        for i in range(len(validated_data)):
+            if sma[i] != 0:
+                bandwidth[i] = (upper_band[i] - lower_band[i]) / sma[i]
+            else:
+                bandwidth[i] = 0.0
+        
+        return self.format_output(bandwidth, input_type, index)
+
+
+class ChandelierExit(BaseIndicator):
+    """
+    Chandelier Exit
+    
+    A trailing stop-loss technique that follows price action.
+    
+    Formula:
+    Long Exit = Highest High(n) - ATR(n) × Multiplier
+    Short Exit = Lowest Low(n) + ATR(n) × Multiplier
+    """
+    
+    def __init__(self):
+        super().__init__("Chandelier Exit")
+    
+    @staticmethod
+    @jit(nopython=True)
+    def _calculate_atr(high: np.ndarray, low: np.ndarray, close: np.ndarray, period: int) -> np.ndarray:
+        """Calculate ATR"""
+        n = len(close)
+        tr = np.full(n, np.nan)
+        atr = np.full(n, np.nan)
+        
+        # Calculate True Range
+        tr[0] = high[0] - low[0]
+        for i in range(1, n):
+            tr[i] = max(high[i] - low[i], 
+                       abs(high[i] - close[i - 1]), 
+                       abs(low[i] - close[i - 1]))
+        
+        # Calculate ATR using SMA
+        for i in range(period - 1, n):
+            atr[i] = np.mean(tr[i - period + 1:i + 1])
+        
+        return atr
+    
+    @staticmethod
+    @jit(nopython=True)
+    def _calculate_chandelier(high: np.ndarray, low: np.ndarray, close: np.ndarray, 
+                             period: int, multiplier: float) -> Tuple[np.ndarray, np.ndarray]:
+        """Numba optimized Chandelier Exit calculation"""
+        n = len(close)
+        long_exit = np.full(n, np.nan)
+        short_exit = np.full(n, np.nan)
+        
+        # Calculate ATR inline
+        tr = np.full(n, np.nan)
+        atr = np.full(n, np.nan)
+        
+        # Calculate True Range
+        tr[0] = high[0] - low[0]
+        for j in range(1, n):
+            hl = high[j] - low[j]
+            hc = abs(high[j] - close[j - 1])
+            lc = abs(low[j] - close[j - 1])
+            tr[j] = max(hl, hc, lc)
+        
+        # Calculate ATR using SMA
+        for j in range(period - 1, n):
+            atr[j] = np.mean(tr[j - period + 1:j + 1])
+        
+        for i in range(period - 1, n):
+            # Highest high and lowest low over period
+            highest_high = np.max(high[i - period + 1:i + 1])
+            lowest_low = np.min(low[i - period + 1:i + 1])
+            
+            # Calculate exits
+            long_exit[i] = highest_high - atr[i] * multiplier
+            short_exit[i] = lowest_low + atr[i] * multiplier
+        
+        return long_exit, short_exit
+    
+    def calculate(self, high: Union[np.ndarray, pd.Series, list],
+                 low: Union[np.ndarray, pd.Series, list],
+                 close: Union[np.ndarray, pd.Series, list],
+                 period: int = 22, multiplier: float = 3.0) -> Union[Tuple[np.ndarray, np.ndarray], Tuple[pd.Series, pd.Series]]:
+        """
+        Calculate Chandelier Exit
+        
+        Parameters:
+        -----------
+        high : Union[np.ndarray, pd.Series, list]
+            High prices
+        low : Union[np.ndarray, pd.Series, list]
+            Low prices
+        close : Union[np.ndarray, pd.Series, list]
+            Closing prices
+        period : int, default=22
+            Period for highest/lowest and ATR calculation
+        multiplier : float, default=3.0
+            ATR multiplier
+            
+        Returns:
+        --------
+        Union[Tuple[np.ndarray, np.ndarray], Tuple[pd.Series, pd.Series]]
+            (long_exit, short_exit) in the same format as input
+        """
+        high_data, input_type, index = self.validate_input(high)
+        low_data, _, _ = self.validate_input(low)
+        close_data, _, _ = self.validate_input(close)
+        
+        high_data, low_data, close_data = self.align_arrays(high_data, low_data, close_data)
+        self.validate_period(period, len(close_data))
+        
+        long_exit, short_exit = self._calculate_chandelier(high_data, low_data, close_data, period, multiplier)
+        
+        results = (long_exit, short_exit)
+        return self.format_multiple_outputs(results, input_type, index)
+
+
+class HistoricalVolatility(BaseIndicator):
+    """
+    Historical Volatility (HV)
+    
+    Measures the standard deviation of logarithmic returns over a specified period.
+    
+    Formula: HV = STDEV(ln(Close/Close[-1]), period) × sqrt(252) × 100
+    """
+    
+    def __init__(self):
+        super().__init__("HV")
+    
+    @staticmethod
+    @jit(nopython=True)
+    def _calculate_hv(data: np.ndarray, period: int, annualize: bool) -> np.ndarray:
+        """Numba optimized Historical Volatility calculation"""
+        n = len(data)
+        result = np.full(n, np.nan)
+        
+        # Calculate log returns
+        log_returns = np.full(n, np.nan)
+        for i in range(1, n):
+            if data[i - 1] > 0 and data[i] > 0:
+                log_returns[i] = np.log(data[i] / data[i - 1])
+        
+        # Calculate rolling standard deviation
+        for i in range(period, n):
+            window = log_returns[i - period + 1:i + 1]
+            valid_returns = window[~np.isnan(window)]
+            
+            if len(valid_returns) >= period:
+                mean_return = np.mean(valid_returns)
+                variance = np.mean((valid_returns - mean_return) ** 2)
+                std_dev = np.sqrt(variance)
+                
+                if annualize:
+                    result[i] = std_dev * np.sqrt(252) * 100  # Annualized percentage
+                else:
+                    result[i] = std_dev * 100  # Percentage
+        
+        return result
+    
+    def calculate(self, data: Union[np.ndarray, pd.Series, list],
+                 period: int = 20, annualize: bool = True) -> Union[np.ndarray, pd.Series]:
+        """
+        Calculate Historical Volatility
+        
+        Parameters:
+        -----------
+        data : Union[np.ndarray, pd.Series, list]
+            Price data (typically closing prices)
+        period : int, default=20
+            Period for volatility calculation
+        annualize : bool, default=True
+            Whether to annualize the volatility
+            
+        Returns:
+        --------
+        Union[np.ndarray, pd.Series]
+            Historical volatility values in the same format as input
+        """
+        validated_data, input_type, index = self.validate_input(data)
+        self.validate_period(period + 1, len(validated_data))
+        
+        result = self._calculate_hv(validated_data, period, annualize)
+        return self.format_output(result, input_type, index)
+
+
+class UlcerIndex(BaseIndicator):
+    """
+    Ulcer Index
+    
+    Measures downside risk by calculating the depth and duration of drawdowns.
+    
+    Formula: UI = sqrt(sum((Close - MaxClose)^2 / MaxClose^2, period) / period) × 100
+    """
+    
+    def __init__(self):
+        super().__init__("Ulcer Index")
+    
+    @staticmethod
+    def _calculate_ulcer_index(data: np.ndarray, period: int) -> np.ndarray:
+        """Optimized Ulcer Index calculation using utils"""
+        return ulcer_index_optimized(data, period)
+    
+    def calculate(self, data: Union[np.ndarray, pd.Series, list],
+                 period: int = 14) -> Union[np.ndarray, pd.Series]:
+        """
+        Calculate Ulcer Index
+        
+        Parameters:
+        -----------
+        data : Union[np.ndarray, pd.Series, list]
+            Price data (typically closing prices)
+        period : int, default=14
+            Period for Ulcer Index calculation
+            
+        Returns:
+        --------
+        Union[np.ndarray, pd.Series]
+            Ulcer Index values in the same format as input
+        """
+        validated_data, input_type, index = self.validate_input(data)
+        self.validate_period(period, len(validated_data))
+        
+        result = self._calculate_ulcer_index(validated_data, period)
+        return self.format_output(result, input_type, index)
+
+
+class STARC(BaseIndicator):
+    """
+    STARC Bands (Stoller Channels)
+    
+    STARC Bands use an SMA and Average True Range to create bands.
+    
+    Formula:
+    Upper Band = SMA + (multiplier × ATR)
+    Lower Band = SMA - (multiplier × ATR)
+    """
+    
+    def __init__(self):
+        super().__init__("STARC Bands")
+    
+    @staticmethod
+    @jit(nopython=True)
+    def _calculate_sma(data: np.ndarray, period: int) -> np.ndarray:
+        """Calculate SMA"""
+        n = len(data)
+        result = np.full(n, np.nan)
+        
+        for i in range(period - 1, n):
+            result[i] = np.mean(data[i - period + 1:i + 1])
+        
+        return result
+    
+    @staticmethod
+    @jit(nopython=True)
+    def _calculate_atr(high: np.ndarray, low: np.ndarray, close: np.ndarray, period: int) -> np.ndarray:
+        """Calculate ATR"""
+        n = len(close)
+        tr = np.full(n, np.nan)
+        atr = np.full(n, np.nan)
+        
+        # Calculate True Range
+        tr[0] = high[0] - low[0]
+        for i in range(1, n):
+            tr[i] = max(high[i] - low[i], 
+                       abs(high[i] - close[i - 1]), 
+                       abs(low[i] - close[i - 1]))
+        
+        # Calculate ATR using SMA
+        for i in range(period - 1, n):
+            atr[i] = np.mean(tr[i - period + 1:i + 1])
+        
+        return atr
+    
+    def calculate(self, high: Union[np.ndarray, pd.Series, list],
+                 low: Union[np.ndarray, pd.Series, list],
+                 close: Union[np.ndarray, pd.Series, list],
+                 ma_period: int = 20, atr_period: int = 15, 
+                 multiplier: float = 2.0) -> Union[Tuple[np.ndarray, np.ndarray, np.ndarray], Tuple[pd.Series, pd.Series, pd.Series]]:
+        """
+        Calculate STARC Bands
+        
+        Parameters:
+        -----------
+        high : Union[np.ndarray, pd.Series, list]
+            High prices
+        low : Union[np.ndarray, pd.Series, list]
+            Low prices
+        close : Union[np.ndarray, pd.Series, list]
+            Closing prices
+        ma_period : int, default=20
+            Period for SMA calculation
+        atr_period : int, default=15
+            Period for ATR calculation
+        multiplier : float, default=2.0
+            ATR multiplier
+            
+        Returns:
+        --------
+        Union[Tuple[np.ndarray, np.ndarray, np.ndarray], Tuple[pd.Series, pd.Series, pd.Series]]
+            (upper_band, middle_line, lower_band) in the same format as input
+        """
+        high_data, input_type, index = self.validate_input(high)
+        low_data, _, _ = self.validate_input(low)
+        close_data, _, _ = self.validate_input(close)
+        
+        high_data, low_data, close_data = self.align_arrays(high_data, low_data, close_data)
+        
+        # Calculate SMA and ATR
+        sma = self._calculate_sma(close_data, ma_period)
+        atr = self._calculate_atr(high_data, low_data, close_data, atr_period)
+        
+        # Calculate bands
+        upper_band = sma + (atr * multiplier)
+        lower_band = sma - (atr * multiplier)
+        
+        results = (upper_band, sma, lower_band)
+        return self.format_multiple_outputs(results, input_type, index)
\ No newline at end of file
